function out = df_dfi_read(varargin)
%df_dfi_read(varargin)  Read digiflow dfi files
if nargin<1
disp('Need filename');
return;
end;

if nargin<2
filename = varargin{1};
end;

if nargin>2
disp('Too many arguments');
return;
end;

out = {};
i = 1;

f =  fopen(filename,'r'); %Check header

header.idFormat = (fread(f,32,'*char'))';
header.Version  = fread(f,1,'*uint32');

if and(header.idFormat ~= 'Tagged floating point image file', header.Version~=0)
disp('Header not valid');
return;
end;
imageno = 0;

while(1)
datatype = fread(f,1,'*uint32');
	if feof(f)
		break;
	end;
nbytes   = fread(f,1,'uint32=>float64');

switch datatype
	case 4097 %8 bit image 0x1001
	image.description                 = '8 bit image';
	image.nx                          = fread(f,1,'*uint32');
	image.ny                          = fread(f,1,'*uint32');

	image                             = check(image.nx*image.ny+8,nbytes,image,dec2hex(datatype));
	image.data                        = double(reshape(fread(f,nbytes-8,'*uint8'),image.nx,image.ny));

	imageno                           = imageno + 1;
	tempstruct.image                  = image;

	case 69633 %hex2dec('11001')
	image.description                 = '8 bit multi-plane image';
	image.nx                          = fread(f,1,'*uint32');
	image.ny                          = fread(f,1,'*uint32');
	image.nz                          = fread(f,1,'*uint32');
	image                             = check(image.nx*image.nz*image.ny+12,nbytes,image,dec2hex(datatype));
	image.data                        = double(reshape(fread(f,nbytes-12,'*uint8'),image.nx,image.ny,image.nz));
	imageno                           = imageno + 1;
	tempstruct.image                  = image;

	case 73729 %hex2dec('12001')
	image.description                 = 'Compressed 8 bit image'
	image.nx                          = fread(f,1,'*uint32');
	image.ny                          = fread(f,1,'*uint32');
	image.nz                          = fread(f,1,'*uint32');
	image.szCompressed                = fread(f,1,'*uint32');
	image                             = check(image.szCompressed +16 ,nbytes,image,dec2hex(datatype));
	image.compresseddata              = fread(f,nbytes-16,'*uint8');
	image.data                        = double(reshape(zlib_uncompress(image.compresseddata),image.nx,image.ny,image.nz));
	imageno                           = imageno + 1;
	tempstruct.image                  = image;


	case 4100 %hex2dec('1004')
	image.description                 = '32 bit image';
	image.nx                          = fread(f,1,'*uint32');
	image.ny                          = fread(f,1,'*uint32');
	image                             = check(4*image.nx*image.ny+8,nbytes,image,dec2hex(datatype));
	image.data                        = double(reshape(fread(f,(nbytes-8)/4,'*float32'),image.nx,image.ny));
	imageno                           = imageno + 1;
	tempstruct.image                  = image;

	case 69636 %hex2dec('11004')
	image.description                 = '32 bit multiplane image';
	image.nx                          = fread(f,1,'*uint32');
	image.ny                          = fread(f,1,'*uint32');
	image.nz                          = fread(f,1,'*uint32');
	image                             = check(4*image.nx*image.nz*image.ny+12,nbytes,image,dec2hex(datatype));
  image.data                        = double(reshape(fread(f,(nbytes-12)/4,'*float32'),image.nx,image.ny,image.nz));
	imageno                           = imageno + 1;
	tempstruct.image                  = image;

	case 73732 %hex2dec('12004')
	image.description                 = '32bit compressed image';
	image.nx                          = fread(f,1,'*uint32');
	image.ny                          = fread(f,1,'*uint32');
	image.nz                          = fread(f,1,'*uint32');
	image.szCompressed                = fread(f,1,'*uint32');
	image                             = check(image.szCompressed +16 ,nbytes,image,dec2hex(datatype));
	image.compresseddata              = fread(f,double(nbytes-16),'*uint8');
	image.data                        = double(reshape(zlib_uncompress(image.compresseddata,32),image.nx,image.ny,image.nz));
	imageno                           = imageno + 1;
	tempstruct.image                  = image;

	case 4104 %hex2dec('1008')
	image.description                 = '64 bit image';
	image.nx                          = fread(f,1,'*uint32');
	image.ny                          = fread(f,1,'*uint32');
	image                             = check(8*image.nx*image.ny+8,nbytes,image,dec2hex(datatype));
	image.data                        = double(reshape(fread(f,(nbytes-8)/8,'*float64'),image.nx,image.ny));
	imageno                           = imageno + 1;
	tempstruct.image                  = image;

	case 69640 %hex2dec('11008')
	image.description                 = '64 bit multiplane image';
	image.nx                          = fread(f,1,'*uint32');
	image.ny                          = fread(f,1,'*uint32');
	image.nz                          = fread(f,1,'*uint32');
	image                             = check(8*image.nx*image.nz*image.ny+12,nbytes,image,dec2hex(datatype));
	image.data                        = double(reshape(fread(f,(nbytes-12)/8,'*float64'),image.nx,image.ny,image.nz));
	imageno                           = imageno + 1;
	tempstruct.image                  = image;

	case 73732 %hex2dec('12004')
	image.description                 = '64bit compressed image';
	image.nx                          = fread(f,1,'*uint32');
	image.ny                          = fread(f,1,'*uint32');
	image.nz                          = fread(f,1,'*uint32');
	image.szCompressed                = fread(f,1,'*uint32');
	image                             = check(image.szCompressed+16,nbytes,image,'12004');
	image.compresseddata              = fread(f,double(nbytes-16),'*uint8');
	image.data                        = double(reshape(zlib_uncompress(image.compresseddata,64),image.nx,image.ny,image.nz));
	imageno                           = imageno + 1;
	tempstruct.image                  = image;

	case 4116 %hex2dec('1014')
	datarange.description             = '32bit data range';
	datarange.rBlack                  = fread(f,1,'*float32');
	datarange.rWhite                  = fread(f,1,'*float32');
	datarange                         = check(8,nbytes,datarange,dec2hex(datatype));
	tempstruct.datarange              = datarange;

	case 4120 %hex2dec('1018')
	datarange.description             = '64bit data range';
	datarange.rBlack                  = fread(f,1,'*float64');
	datarange.rWhite                  = fread(f,1,'*float64');
	datarange                         = check(16,nbytes,datarange,dec2hex(datatype));
	tempstruct.datarange              = datarange;

	case 4352 %hex2dec('1100')
	rescale.description               = 'Rescale image';
	rescale.nxWant                    = fread(f,1,'*uint32');
	rescale.nyWant                    = fread(f,1,'*uint32');
	rescale.method                    = fread(f,1,'*uint32');
	rescale                           = check(12,nbytes,rescale,dec2hex(datatype));
	tempstruct.rescale                = rescale;

	case 4353 %hex2dec('1101')
	rescalerect.description           = 'Rescale image rectangle';
	rescalerect.nxwant                = fread(f,1,'*uint32');
	rescalerect.nyWant                = fread(f,1,'*uint32');
	rescalerect.method                = fread(f,1,'*uint32');
	rescalerect.userectangle          = fread(f,1,'*uint32');
	rescalerect.rectangle.xMin        = fread(f,1,'*uint32');
	rescalerect.rectangle.yMin        = fread(f,1,'*uint32');
	rescalerect.rectangle.xMax        = fread(f,1,'*uint32');
	rescalerect.rectangle.yMax        = fread(f,1,'*uint32');
	rescalerect                       = check(32,nbytes,rescalerect,dec2hex(datatype));
	tempstruct.rescalerect            = rescalerect;

	case 8192 %hex2dec('2000')
	colourscheme.description          = 'Colour scheme';
	coloursceme.colours = [];
	colourscheme.colours              = reshape(fread(f,768,'*uint8'),3,256);
	colourscheme                      = check(768,nbytes,colourscheme,dec2hex(datatype));
	tempstruct.colourscheme           = colourscheme;

	case 8193 %hex2dec('2001')
	colourscheme.description          = 'Colour scheme';
	colourscheme.name                 = fread(f,64,'*char')';
	colourscheme                      = check(64,nbytes,colourscheme,dec2hex(datatype));
	tempstruct.colourscheme           = colourscheme;

	case 8194 %hex2dec('2002')
	colourscheme.description          = 'Colour scheme';
	colourscheme.namevarlen           = fread(f,1,'*uint32');
	colourscheme                      = check(colourscheme.namevarlen + 4,nbytes,colourscheme,dec2hex(datatype));
	colourscheme.namevar              = fread(f,nbytes-4,'*char')';
	tempstruct.colourscheme           = colourscheme;

	case 12288 %hex2dec('3000')
	comment.description               = 'Description';
	comment.descr                     = fread(f,512,'*char')';
	comment                           = check(512,nbytes,comment,dec2hex(datatype));
	tempstruct.comment                = comment;

	case 12289 %hex2dec('3001')
	comment.description               = 'User comments';
	comment.nbytes                    = fread(f,1,'*uint32');
	comment                           = check(comment.nbytes+4,nbytes,comment,dec2hex(datatype));
	comment.comment                   = fread(f,double(comment.nbytes),'*char')';
	tempstruct.comment                = comment;

	case 12290 %hex2dec('3002')
	process.description               = 'Creating process';
	process.nbytes                    = fread(f,1,'*uint32');
	process                           = check(process.nbytes + 4,nbytes,process,dec2hex(datatype));
	process.descr                     = fread(f,double(process.nbytes),'*char')';
	tempstruct.process                = process;

	case 12291 %hex2dec('3003')
	creator.description               = 'Creator details';
	creator                           = check(304,nbytes,creator,dec2hex(datatype));
	creator.digiflow                  = fread(f,32,'*char')';
	creator.builddate                 = fread(f,16,'*char')';
	creator.licencetype               = fread(f,16,'*char')';
	creator.nameuser                  = fread(f,32,'*char')';
	creator.namecomputer              = fread(f,32,'*char')';
	creator.namedomain                = fread(f,32,'*char')';
	creator.guiduser                  = fread(f,32,'*char')';
	creator.macaddress                = fread(f,48,'*char')';
	creator.ipaddress                 = fread(f,64,'*char')';
	tempstruct.creator                = creator;

	case 12312 %hex2dec('3018')
	time.description                  = 'Image time';
	time                              = check(32,nbytes,time,dec2hex(datatype));
	time.iframe                       = fread(f,1,'*uint32');
	time.reserved                     = fread(f,1,'*uint32');
	time.time                         = fread(f,1,'*float64');
	time.tstep                        = fread(f,1,'*float64');
	time.tfirst                       = fread(f,1,'*float64');
	tempstruct.time                   = time;

	case 16392 %hex2dec('4008')
	coords.description                = 'Image coordinates';
	coords                            = check(132,nbytes,coords,dec2hex(datatype));
	coords.kind                       = fread(f,1,'*uint32');
	coords.xworldperpixel             = fread(f,1,'*float64');
	coords.yworldperpixel             = fread(f,1,'*float64');
	coords.xoriginworld               = fread(f,1,'*float64');
	coords.yoriginworld               = fread(f,1,'*float64');
	coords.xunits                     = fread(f,16,'*char')';
	coords.yunits                     = fread(f,16,'*char')';
	coords.originalname               = fread(f,64,'*char')';
	tempstruct.coords                 = coords;

	case 16648 %hex2dec('4108')

	implane.description               = 'Image plane details';
	implane.nplanes                   = fread(f,1,'*uint32');
        implane                           = check(100*implane.nplanes+4,nbytes,implane,dec2hex(datatype));
        implane.contains = [];
	implane.descr    = {};
	implane.parama   = [];
	implane.paramb   = [];
	implane.paramc   = [];
	implane.paramd   = [];
	implane.dunno    = {};
	for counter_i=1:implane.nplanes
	implane.contains  = [implane.contains; fread(f,1,'*uint32' )];
	implane.descr     = [implane.descr;    fread(f,32,'*char'  )'];
	implane.parama    = [implane.parama;   fread(f,1,'*float64')];
	implane.paramb    = [implane.paramb;   fread(f,1,'*float64')];
	implane.paramc    = [implane.paramc;   fread(f,1,'*float64')];
	implane.paramd    = [implane.paramd;   fread(f,1,'*float64')];
	implane.dunno     = [implane.dunno;    fread(f,32,'*char')'];
	end;
	tempstruct.implane = implane;
	otherwise
	disp(datatype);

	end;
	tempstruct.header                 = header;

if imageno         == 1
out                = tempstruct;
elseif imageno     == 2
tempim             = out.image;
out.image          = {};
out.image{1}       = tempim;
out.image{2}       = tempstruct.image;
elseif imageno > 2
out.image{imageno} = tempstruct.image;
end;


%%if (exist('image')) clear('image'); end;
%if (exist('datarange')) clear('datarange'); end;
%if (exist('rescale')) clear('rescale'); end;
%if (exist('rescalerect')) clear('rescalerect'); end;
%if (exist('colourscheme')) clear('colourscheme'); end;
%if (exist('comment')) clear('comment'); end;
%if (exist('process')) clear('process'); end;
%if (exist('creator')) clear('creator'); end;
%if (exist('time')) clear('time'); end;
%if (exist('coords')) clear('coords'); end;
%if (exist('implane')) clear('implane'); end;




%image
%datarange
%rescale
%rescalerect
%colourscheme
%comment
%process
%creator
%time
%coords
%implane

end;
fclose(f);
return;

function out=check(a,b,str,datatype)
if(a==b)

else
str.corrupt = 'corrupt';
fprintf('Datatype %s is corrupt arg1 = %f  arg2 = %f\n',datatype,a,b);
end;
out = str;
return;

